package fullenv

import (
	"fmt"
	"strconv"
	"testing"
	"time"

	"bitbucket.org/orus-io/go-xbus"
)

func NewCounterReceiver(t *testing.T, limit int) *CounterReceiver {
	return &CounterReceiver{
		t:              t,
		limit:          limit,
		receivedAll:    make(chan bool),
		missingnumbers: make(map[int]bool),
	}
}

type CounterReceiver struct {
	actor *xbus.Actor
	sub   *xbus.Subscription
	t     *testing.T

	offset int
	limit  int

	nextExpected int

	missingnumbers map[int]bool
	extranumbers   []int

	receivedAll chan bool
}

func (cr *CounterReceiver) Service(actor *xbus.Actor) xbus.ActorService {
	fmt.Println("Service", cr, actor)
	cr.actor = actor
	return cr
}

func (cr *CounterReceiver) Startup() error {
	fmt.Println(cr, cr.actor)
	sub, err := cr.actor.Subscribe(func(er *xbus.EnvelopeReceiver) {
		envelope, err := er.Complete(time.Millisecond)
		n, err := strconv.Atoi(string(envelope.Events[0].Items[0]))
		if err != nil {
			cr.t.Error(err)
			cr.actor.ProcessingError(er.Context(), er.ID, err.Error())
		} else {
			cr.actor.GetClient().Log.Noticef("Received %d", n)
			cr.actor.ProcessingSuccess(er.Context(), er.ID)
		}
		if n < cr.offset || n > cr.limit {
			cr.extranumbers = append(cr.extranumbers, n)
		} else if n > cr.nextExpected {
			for i := cr.nextExpected; i < n; i++ {
				cr.missingnumbers[i] = true
			}
			cr.nextExpected = n + 1
		} else if n < cr.nextExpected {
			delete(cr.missingnumbers, n)
		} else {
			cr.nextExpected = n + 1
		}
		if cr.nextExpected > cr.limit && len(cr.missingnumbers) == 0 {
			cr.receivedAll <- true
		}
	})
	if err == nil {
		cr.sub = sub
	}
	return err
}

func (cr *CounterReceiver) Shutdown() error {
	err := cr.sub.Unsubscribe()
	cr.sub = nil
	return err
}

// TestCounterRelay makes sure counter_relay env works properly
// under normal conditions
func TestCounterRelay(t *testing.T) {
	env := NewFullEnvForTest(t, "counter-relay", "counter_relay.yaml")

	if err := env.Init(); err != nil {
		t.Fatal(err)
	}

	defer env.Close()

	cr := NewCounterReceiver(t, 200)
	env.RegisterActorService("test.receivecounter", cr.Service)

	if err := env.Startup(); err != nil {
		t.Fatal(err)
	}

	select {
	case <-cr.receivedAll:
		t.Log("Everything's alright")
	case <-time.After(60 * time.Second):
		t.Errorf("Reception timeout")
		t.Errorf("Next number expected: %d", cr.nextExpected)
		t.Errorf("Missing numbers: %v", cr.missingnumbers)
	}
}
