package fullenv

import (
	"time"

	"bitbucket.org/orus-io/go-xbus"
	"bitbucket.org/orus-io/go-xbus/api"
)

// NewEmitter initializes a Emitter
func NewEmitter(logger xbus.Logger) *Emitter {
	return &Emitter{
		log: logger,
	}
}

// Emitter is an emitter actor for FullEnv
type Emitter struct {
	Actor *xbus.Actor
	log   xbus.Logger
}

// Service is a Actor factory
func (e *Emitter) Service(actor *xbus.Actor) xbus.ActorService {
	e.Actor = actor
	return e
}

// Startup ...
func (e *Emitter) Startup() error {
	return nil
}

// Shutdown ...
func (e *Emitter) Shutdown() error {
	return nil
}

// NewConsumer initializes a Consumer
func NewConsumer(logger xbus.Logger) *Consumer {
	return &Consumer{
		log: logger,
	}
}

// Consumer is an emitter actor for FullEnv
type Consumer struct {
	Actor *xbus.Actor
	log   xbus.Logger

	sub *xbus.Subscription

	Envelopes []*api.Envelope
}

// Service is a Actor factory
func (c *Consumer) Service(actor *xbus.Actor) xbus.ActorService {
	c.Actor = actor
	return c
}

// Startup ...
func (c *Consumer) Startup() error {
	sub, err := c.Actor.Subscribe(func(er *xbus.EnvelopeReceiver) {
		go func() {
			env, err := er.Complete(time.Second)
			if err != nil {
				c.log.Errorf("Error receiving envelope: %s", err)
				return
			}
			c.Envelopes = append(c.Envelopes, env)
		}()
	})
	if err != nil {
		return err
	}
	c.sub = sub
	return nil
}

// Shutdown ...
func (c *Consumer) Shutdown() error {
	return c.sub.Unsubscribe()
}
