// Copyright 2017 Orus.io. All rights reserved.

package broker

import (
	"fmt"
	"strings"

	"github.com/m4rw3r/uuid"

	"bitbucket.org/orus-io/go-xbus/api"
)

func parseActorSubject(subject string) (actorID uuid.UUID, action string, err error) {
	splitted := strings.SplitN(subject, ".", 4)
	if len(splitted) < 4 {
		err = fmt.Errorf(
			"Invalid subject '%s'. Expected at least 4 parts, got %d",
			subject, len(splitted))
		return
	}
	if splitted[0] != "xbus" || splitted[1] != "actor" {
		err = fmt.Errorf(
			"Invalid subject '%s'. It must starts with 'xbus.actor' %s.%s",
			subject, splitted[0], splitted[1])
		return
	}
	actorID, err = uuid.FromString(splitted[2])
	action = splitted[3]
	return
}

func (s *Service) handleActorLog(subject, reply string, req api.ActorLogReq) {
	var rep = api.Response{Status: api.StatusOK}
	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	actorID, action, err := parseActorSubject(subject)

	if err != nil {
		panic(err)
	}
	if action != "log" {
		panic("Invalid subject action: " + action)
	}

	if !req.EnvelopeID.IsZero() {
		_, err = s.Opts.ProcessManager.Get(req.Context.ProcessID)
		if err != nil {
			rep = api.ClientError("%s", err)
			return
		}
	}

	for _, msg := range req.Messages {
		s.Opts.LogStorage.Log(api.LogEntry{
			EnvelopeID: req.EnvelopeID,
			ActorID:    actorID,
			NodeID:     req.Context.NodeID,
			ProcessID:  req.Context.ProcessID,
			Message:    msg,
		})
	}
}

func (s *Service) handleActorProcessingEnd(subject, reply string, req api.ActorProcessingEndReq) {
	var rep = api.Response{Status: api.StatusOK}
	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	actorID, action, err := parseActorSubject(subject)

	if err != nil {
		panic(err)
	}
	if action != "processingend" {
		panic("Invalid subject action: " + action)
	}

	p, err := s.Opts.ProcessManager.Get(req.Context.ProcessID)
	if err != nil {
		rep = api.ClientError("%s", err)
		return
	}

	for _, msg := range req.Messages {
		s.Opts.LogStorage.Log(api.LogEntry{
			EnvelopeID: req.EnvelopeID,
			ActorID:    actorID,
			NodeID:     req.Context.NodeID,
			ProcessID:  req.Context.ProcessID,
			Message:    msg,
		})
	}

	p.ActorProcessingEnd(req.EnvelopeID, actorID, req.Context.NodeID, req.Result)
}
