// Copyright 2016 Orus.io. All rights reserved.

package broker

import (
	"github.com/m4rw3r/uuid"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/process"
	"bitbucket.org/orus-io/xbus/services"
	"bitbucket.org/orus-io/xbus/storage"
)

const (
	// BrokerQueue is the name we use to queueSub to a queue with the broker
	BrokerQueue = "broker_queue"
)

// Options is used to create a Service
type Options struct {
	services.Options

	ActorStorage    storage.ActorStorage
	EnvelopeStorage storage.EnvelopeStorage
	LogStorage      storage.LogStorage
	ProcessManager  *process.Manager
}

// NewService returns a new Service
func (o Options) NewService() *Service {
	return &Service{
		BaseService:    services.NewBaseService("broker", o.Options),
		Opts:           o,
		processManager: o.ProcessManager,
	}
}

// New returns a new Service
func New(options Options) *Service {
	return options.NewService()
}

// Service is the central node of all envelope exchanges
type Service struct {
	services.BaseService
	Opts Options

	processManager *process.Manager
}

// Startup initialize all the subscriptions
func (s *Service) Startup() error {
	actorOutputSubjects := api.Msgbox + ".*.output.*"
	actorLogSubjects := api.ActorLog(uuid.UUID{})
	actorProcessingEndSubjects := api.ActorProcessingEnd(uuid.UUID{})
	if err := s.SubscribeAll(
		services.QueueSubscribe(actorOutputSubjects, BrokerQueue, s.handleIncomingEnvelope),
		services.QueueSubscribe(actorLogSubjects, BrokerQueue, s.handleActorLog),
		services.QueueSubscribe(actorProcessingEndSubjects, BrokerQueue, s.handleActorProcessingEnd),
	); err != nil {
		return err
	}
	s.Log.Noticef("listening for incoming envelopes")
	return nil
}
