// Copyright 2016 Orus.io. All rights reserved.

package sentinel

import (
	"strings"

	"github.com/m4rw3r/uuid"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/services"
	"bitbucket.org/orus-io/xbus/storage"
)

const (
	// ServiceName name of the service
	ServiceName = "sentinel"

	// QueueName queue name for queuesubscribes
	QueueName = "xbus.services." + ServiceName
)

// Options is used to create a 'processstate' service
type Options struct {
	services.Options

	ActorStorage storage.ActorStorage
}

// NewService creates a Service
func (o Options) NewService() *Service {
	return &Service{
		BaseService: services.NewBaseService(ServiceName, o.Options),
		o:           o,
	}
}

// Service is in charge of emitting process status events to various subjects
type Service struct {
	services.BaseService

	o Options
}

// Startup starts the service
func (s *Service) Startup() error {
	if err := s.SubscribeAll(
		services.QueueSubscribe(api.ActorReadySubject(uuid.UUID{}), QueueName, s.handleActorReady),
	); err != nil {
		return err
	}
	return nil
}

func (s *Service) handleActorReady(subject string, msg api.ActorReadyMsg) {
	defer s.HandlePanics()

	// read actor ID in the subject
	actorID := uuid.MustFromString(strings.Split(subject, ".")[2])

	// signal the actor was just seen
	s.o.ActorStorage.JustSeen(actorID)
}
