// Copyright 2016 Orus.io. All rights reserved.

package api

import (
	"github.com/m4rw3r/uuid"
)

const (
	// ActorPrefix is the prefix of actor API subjects
	ActorPrefix = XbusAPIPrefix + "actor."

	// ActorProcessingEndPostfix is the postfix for the processingend subject
	ActorProcessingEndPostfix = ".processingend"

	// ActorLogPostfix is the postfix for the processingend subject
	ActorLogPostfix = ".log"
)

// ActorProcessingEnd is for signaling the end of an envelope
// processing by an actor.
// If a node closes all its outputs, the signal is done automatically.
// In all other cases (errors, need to close all outputs but the one
// used, or consumers), it is required.
func ActorProcessingEnd(actorID uuid.UUID) string {
	var sActor string
	if actorID.IsZero() {
		sActor = "*"
	} else {
		sActor = actorID.String()
	}
	return ActorPrefix + sActor + ActorProcessingEndPostfix
}

// ActorLog returns the subject for ActorLog calls
func ActorLog(actorID uuid.UUID) string {
	return ActorPrefix + UUIDToStringOrWildcard(actorID) + ActorLogPostfix
}

// ActorLogReq is the type of request to send to ActorLog subject
type ActorLogReq struct {
	EnvelopeID uuid.UUID       `json:"envelope_id"`
	Context    EnvelopeContext `json:"context"`
	Messages   []LogMessage    `json:"messages"`
}

// ActorProcessingResultType tells if an envelope processing was a sucess or not
//go:generate enum-marshaler -type ActorProcessingResultType -prefix ActorProcessing -lower -tests
type ActorProcessingResultType int

const (
	// ActorProcessingSuccess is when the node successfully processed the envelope
	ActorProcessingSuccess ActorProcessingResultType = iota

	// ActorProcessingError is when the node encountered an error processing
	// the envelope
	ActorProcessingError
)

// ActorProcessingEndReq is the message sent to ActorProcessingEnd
type ActorProcessingEndReq struct {
	EnvelopeID uuid.UUID                 `json:"envelope_id"`
	Context    EnvelopeContext           `json:"context"`
	Result     ActorProcessingResultType `json:"result"`
	Messages   []LogMessage              `json:"messages"`
}

// ActorProcessingEndRep is the response from ActorProcessingEnd calls
type ActorProcessingEndRep struct {
	Response
}
