// Copyright 2016 Orus.io. All rights reserved.

package api

import (
	"fmt"
)

const (
	// XbusAPIPrefix is the common prefix of all the nats subjects of the
	// xbus API.
	XbusAPIPrefix = "xbus."
)

// Status is used for response status
//go:generate enum-marshaler -type Status -prefix Status -tests
type Status int

const (
	// StatusOK is the status of responses without error
	StatusOK Status = iota + 1
	// StatusServerError is the status of responses with a server error
	StatusServerError
	// StatusClientError is the status of responses with an error due to the client input
	StatusClientError
)

const (
	// ActionCreate is the subject suffix for create operations
	ActionCreate = "create"
	// ActionUpdate is the subject suffix for update operations
	ActionUpdate = "update"
	// ActionDelete is the subject suffix for delete operations
	ActionDelete = "delete"
	// ActionQuery is the subject suffix for query operations
	ActionQuery = "query"

	// ServicesPrefix is the prefix for all service specific subjects
	ServicesPrefix = XbusAPIPrefix + "services."
)

// EmptyRequest is for REQ/REP without parameters
type EmptyRequest struct{}

// Response is the base response type. All responses should embed this struct
type Response struct {
	Status        Status `json:"status,omitempty"`
	StatusMessage string `json:"status_message,omitempty"`
}

// ServerError returns a server error Response
func ServerError(err error) Response {
	return Response{
		Status:        StatusServerError,
		StatusMessage: err.Error(),
	}
}

// ClientError returns a client error Response
func ClientError(msg string, fmtArgs ...interface{}) Response {
	return Response{
		Status:        StatusClientError,
		StatusMessage: fmt.Sprintf(msg, fmtArgs...),
	}
}
