package api

import (
	"github.com/m4rw3r/uuid"
)

// GraphStatus is the status of a graph
//go:generate enum-marshaler -type GraphStatus -prefix Graph -lower -tests
type GraphStatus int

const (
	// GraphDraft is for graphs that still can be modified
	GraphDraft GraphStatus = iota

	// GraphActive is for graphs that can be triggered by incoming envelopes
	GraphActive

	// GraphInactive is for graphs that are no longer (or not yet) actives
	GraphInactive
)

// GraphInfo represents the information about a graph
type GraphInfo struct {
	ID      uuid.UUID
	Name    string
	Version string
	Status  GraphStatus
}

// ControlGraphPrefix prefix for control graph
const ControlGraphPrefix = ControlPrefix + "graph."

// ControlGraphQuery api to retrive the graphs
const ControlGraphQuery = ControlGraphPrefix + ActionQuery

// ControlGraphQueryReq ...
type ControlGraphQueryReq struct {
	Name string
}

// ControlGraphQueryRep ...
type ControlGraphQueryRep struct {
	Response

	GraphList []GraphInfo
}

// ControlGraphSave saves a draft version of a graph
const ControlGraphSave = ControlGraphPrefix + "save"

// ControlGraphSaveReq ...
type ControlGraphSaveReq struct {
	Info  GraphInfo
	Graph string
}

// ControlGraphSaveRep ...
type ControlGraphSaveRep struct {
	Response
	Info               GraphInfo
	Graph              string
	ValidationMessages []string
}

// ControlGraphSetStatus changes the status of a graph
const ControlGraphSetStatus = ControlGraphPrefix + "setstatus"

// ControlGraphSetStatusReq is the arguments for ControlGraphSetStatus
type ControlGraphSetStatusReq GraphInfo

// ControlGraphSetStatusRep is the reply of ControlGraphSetStatus
type ControlGraphSetStatusRep struct {
	Response
	Info               GraphInfo
	ValidationMessages []string
}
