package demo

import (
	"fmt"
	"time"

	"bitbucket.org/orus-io/go-xbus"
	"bitbucket.org/orus-io/go-xbus/api"
)

// NewCounterEmitter is a CounterEmitter factory
func NewCounterEmitter(actor *xbus.Actor) xbus.ActorService {
	return &CounterEmitter{
		actor:    actor,
		interval: actor.GetIntSettingD("interval", 1),
		offset:   actor.GetIntSettingD("offset", 0),
		limit:    actor.GetIntSettingD("limit", 100),
		count:    actor.GetIntSettingD("offset", 0),
	}
}

// CounterEmitter is an emitter that counts from
type CounterEmitter struct {
	actor    *xbus.Actor
	interval int
	offset   int
	limit    int
	count    int
	quit     chan bool
}

// Startup starts a routine that send all the counter values
func (c *CounterEmitter) Startup() error {
	if c.count > c.limit {
		return nil
	}
	c.quit = make(chan bool)
	go func() {
		for {
			select {
			case <-time.After(time.Duration(c.interval) * time.Millisecond):
				message := fmt.Sprintf("%d", c.count)
				if _, err := c.actor.SendItems(
					"default", "demo.count",
					[]api.Item{api.Item(message)},
				); err != nil {
					c.actor.GetClient().Log.Errorf("send failed: %s", err)
				}
				c.actor.GetClient().Log.Noticef("sent %d", c.count)
				c.count++
				if c.count > c.limit {
					return
				}
			case <-c.quit:
				return
			}
		}
	}()
	return nil
}

// Shutdown stops the ticker
func (c *CounterEmitter) Shutdown() error {
	close(c.quit)
	return nil
}

func init() {
	xbus.RegisterActorService("demo.counter", NewCounterEmitter)
}
